Package: gofmalm
Type: Package
Title: Goodness-of-Fit Tests for Type-II Censored Samples via the
        Malmquist Transformation
Version: 0.1.0
Authors@R: c(
    person("Shikhar", "Tyagi", email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844")),
    person("Arvind", "Pandey", email = "arvindmzu@gmail.com", role = "aut"),
    person("Bhupendra", "Singh", email = "bhupendra.rana@gmail.com", role = "aut"),
    person("Vrijesh", "Tripathi", email = "vrijesh.tripathi@uwi.edu", role = "aut"))
Description: Goodness-of-fit tests for an arbitrary user-specified continuous
    distribution under Type-II right- or left-censoring. Implements the
    transformation-based method of Lin, Huang and Balakrishnan (2008)
    <doi:10.1109/TR.2008.2005860>, which uses a property of order statistics
    due to Malmquist (1950) to convert an r-out-of-n Type-II censored uniform
    sample into a complete sample of size r, alongside the earlier
    transformation of Michael and Schucany (1979)
    <doi:10.1080/00401706.1979.10489813>. Also implements the direct
    (untransformed) censored-sample statistics of Barr and Davidson (1973)
    <doi:10.1080/00401706.1973.10489108> and Pettitt and Stephens (1976)
    <doi:10.1093/biomet/63.2.291>, and the modified-statistic
    maximum-likelihood procedure of Chen and Balakrishnan (1995) for testing
    composite hypotheses. General background on empirical-distribution-function
    goodness-of-fit methods follows D'Agostino and Stephens (1986,
    ISBN:982-0-8247-7487-5).
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
Depends: R (>= 4.1.0)
Imports: stats, graphics
Suggests: testthat (>= 3.0.0)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-22 11:04:04 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Arvind Pandey [aut],
  Bhupendra Singh [aut],
  Vrijesh Tripathi [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-30 17:40:08 UTC
Built: R 4.6.0; ; 2026-07-30 20:54:39 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,gofmalm)
S3method(print,gofmalm)
S3method(summary,gofmalm)
export(fit_mle)
export(gof_censored)
export(pvalue_distribution_free)
export(pvalue_parametric_bootstrap)
export(stat_composite_mle)
export(stat_direct_censored)
export(stat_edf_censored)
export(transform_malmquist)
export(transform_ms)

# gofmalm: Goodness-of-Fit Tests for Type-II Censored Samples via Malmquist Transformation

`gofmalm` provides goodness-of-fit (GOF) tests for arbitrary user-specified continuous parametric distributions under Type-II right- or left-censoring.

## Features

- Implements the transformation-based goodness-of-fit testing method of **Lin, Huang, and Balakrishnan (2008)** using the **Malmquist (1950)** transformation.
- Implements the earlier transformation of **Michael and Schucany (1979)**.
- Computes all 9 empirical distribution function (EDF) test statistics ($D_{r,n}, G_{r,n}, T_{r,n}, ^*W^2_{r,n}, W^2_{r,n}, A^2_{r,n}, U^2_{r,n}, ^TA^2_r, ^{*T}A^2_r$).
- Computes direct untransformed censored EDF statistics of **Barr & Davidson (1973)** and **Pettitt & Stephens (1976)** ($_{2}W^2_{r,n}, _{2}A^2_{r,n}$).
- Supports simple hypotheses (known parameters, distribution-free simulation) and composite hypotheses (unknown parameters estimated via MLE, parametric bootstrap).
- Full support for Type-II left-censoring via quantile reflection.

## Installation

You can install `gofmalm` from the source package tarball:

```r
install.packages("gofmalm_0.1.0.tar.gz", repos = NULL, type = "source")
```

## Quick Example

```r
library(gofmalm)

# Example 1: Testing Uniform(0,1) with r = 5, n = 10 (simple hypothesis)
data_unif <- c(0.03, 0.06, 0.10, 0.11, 0.13)
res1 <- gof_censored(
  data = data_unif,
  n = 10,
  pdf = dunif,
  cdf = punif,
  sf = function(x, p) 1 - punif(x),
  estimate = FALSE,
  pvalue_method = "distribution_free",
  nsim = 1000,
  seed = 123
)
print(res1)
```

## References

- Lin, C.-T., Huang, Y.-L., & Balakrishnan, N. (2008). A New Method for Goodness-of-Fit Testing Based on Type-II Right Censored Samples. *IEEE Transactions on Reliability*, 57(4), 633–642. <doi:10.1109/TR.2008.2005860>
- Michael, J. R., & Schucany, W. R. (1979). A New Approach to Testing Goodness of Fit for Censored Samples. *Technometrics*, 21(4), 435–441. <doi:10.1080/00401706.1979.10489813>
- Pettitt, A. N., & Stephens, M. A. (1976). Modified Cramér-von Mises statistics for censored data. *Biometrika*, 63(2), 291–298. <doi:10.1093/biomet/63.2.291>

#' Compute Distribution-Free p-values for Simple Hypotheses
#'
#' Obtains p-values for EDF goodness-of-fit statistics via Monte Carlo simulation
#' from Uniform(0,1) order statistics.
#'
#' @param u_obs Observed order statistics u_1:n < ... < u_r:n.
#' @param n Total sample size.
#' @param r Number of observed failures.
#' @param censoring Type of censoring: \code{"right"} or \code{"left"}.
#' @param nsim Number of Monte Carlo simulation replicates (default 2000).
#' @param seed Optional random seed.
#'
#' @return Named numeric vector of empirical p-values for each statistic.
#' @export
pvalue_distribution_free <- function(u_obs,
                                     n,
                                     r,
                                     censoring = c("right", "left"),
                                     nsim = 2000,
                                     seed = NULL) {
  censoring <- match.arg(censoring)
  if (!is.null(seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv)) {
      old_seed <- get(".Random.seed", envir = .GlobalEnv)
      on.exit(assign(".Random.seed", old_seed, envir = .GlobalEnv), add = TRUE)
    }
    set.seed(seed)
  }

  obs_edf <- stat_edf_censored(u_obs, n)
  obs_dir <- stat_direct_censored(u_obs, n)
  obs_all <- c(obs_edf, obs_dir)

  sim_stats <- matrix(NA_real_, nrow = nsim, ncol = length(obs_all))
  colnames(sim_stats) <- names(obs_all)

  for (b in 1:nsim) {
    unif_sample <- sort(stats::runif(n))
    if (censoring == "right") {
      u_sim <- unif_sample[1:r]
    } else {
      u_sim <- unif_sample[(n - r + 1):n]
      u_sim <- 1 - u_sim[r:1]
    }

    edf_b <- stat_edf_censored(u_sim, n)
    dir_b <- stat_direct_censored(u_sim, n)
    sim_stats[b, ] <- c(edf_b, dir_b)
  }

  p_vals <- numeric(length(obs_all))
  names(p_vals) <- names(obs_all)

  for (j in seq_along(obs_all)) {
    obs_val <- obs_all[j]
    if (is.na(obs_val)) {
      p_vals[j] <- NA_real_
    } else {
      p_vals[j] <- mean(sim_stats[, j] >= obs_val, na.rm = TRUE)
    }
  }

  p_vals
}

# Helper to sample from arbitrary CDF
q_generic <- function(p_vec, cdf, params) {
  check_zero <- tryCatch(eval_func(cdf, 0, params), error = function(e) NA_real_)
  is_nonneg <- !is.na(check_zero) && check_zero >= 0 && check_zero <= 0.05

  sapply(p_vec, function(u) {
    low <- if (is_nonneg) 0 else -10
    high <- 10
    step <- 10

    if (!is_nonneg) {
      count <- 0
      while (tryCatch(eval_func(cdf, low, params), error = function(e) 1) > u && count < 50) {
        low <- low - step
        step <- step * 2
        count <- count + 1
      }
    }

    step <- 10
    count <- 0
    while (tryCatch(eval_func(cdf, high, params), error = function(e) 0) < u && count < 50) {
      high <- high + step
      step <- step * 2
      count <- count + 1
    }

    res <- tryCatch(
      stats::uniroot(function(x) eval_func(cdf, x, params) - u, lower = low, upper = high)$root,
      error = function(e) NA_real_
    )
    res
  })
}

#' Compute Parametric Bootstrap p-values for Composite Hypotheses
#'
#' Obtains p-values for composite hypothesis tests where parameters are estimated from data.
#'
#' @param data Observed data vector of length r.
#' @param n Total sample size.
#' @param pdf Density function.
#' @param cdf Distribution function.
#' @param sf Survival function.
#' @param params_hat Estimated parameter list from observed data.
#' @param censoring Type of censoring.
#' @param transform Transformation method (\code{"ms"}, \code{"malmquist"}, or \code{"both"}).
#' @param nsim Number of bootstrap replicates (default 2000).
#' @param seed Optional random seed.
#' @param optim_control Optional control list passed to \code{optim}.
#'
#' @return A list containing empirical p-values and number of valid replicates.
#' @export
pvalue_parametric_bootstrap <- function(data,
                                        n,
                                        pdf,
                                        cdf,
                                        sf = NULL,
                                        params_hat,
                                        censoring = c("right", "left"),
                                        transform = c("both", "malmquist", "ms"),
                                        nsim = 2000,
                                        seed = NULL,
                                        optim_control = list()) {
  censoring <- match.arg(censoring)
  transform <- match.arg(transform)

  if (!is.null(seed)) {
    if (exists(".Random.seed", envir = .GlobalEnv)) {
      old_seed <- get(".Random.seed", envir = .GlobalEnv)
      on.exit(assign(".Random.seed", old_seed, envir = .GlobalEnv), add = TRUE)
    }
    set.seed(seed)
  }

  r <- length(data)

  # Compute observed composite statistics
  calc_comp <- function(trans_type) {
    stat_composite_mle(data = data, n = n, pdf = pdf, cdf = cdf, sf = sf,
                       params = params_hat, censoring = censoring,
                       transform = trans_type, optim_control = optim_control)
  }

  obs_ms <- if (transform %in% c("both", "ms")) calc_comp("ms") else NULL
  obs_mal <- if (transform %in% c("both", "malmquist")) calc_comp("malmquist") else NULL

  boot_ms_W2 <- numeric(nsim)
  boot_ms_A2 <- numeric(nsim)
  boot_mal_W2 <- numeric(nsim)
  boot_mal_A2 <- numeric(nsim)

  valid_count <- 0

  for (b in 1:nsim) {
    u_rand <- sort(stats::runif(n))
    x_sim <- q_generic(u_rand, cdf, params_hat)

    if (any(is.na(x_sim))) next

    if (censoring == "right") {
      data_b <- x_sim[1:r]
    } else {
      data_b <- x_sim[(n - r + 1):n]
    }

    # Re-estimate MLE on bootstrap sample
    fit_b <- tryCatch(
      fit_mle(data = data_b, n = n, pdf = pdf, cdf = cdf, sf = sf,
              params = params_hat, censoring = censoring, optim_control = optim_control),
      error = function(e) NULL
    )

    if (is.null(fit_b) || fit_b$convergence != 0) next

    # Compute bootstrap composite statistics
    comp_b_ms <- tryCatch(
      stat_composite_mle(data = data_b, n = n, pdf = pdf, cdf = cdf, sf = sf,
                         params = fit_b$par, censoring = censoring,
                         transform = "ms", optim_control = optim_control),
      error = function(e) NULL
    )

    comp_b_mal <- tryCatch(
      stat_composite_mle(data = data_b, n = n, pdf = pdf, cdf = cdf, sf = sf,
                         params = fit_b$par, censoring = censoring,
                         transform = "malmquist", optim_control = optim_control),
      error = function(e) NULL
    )

    if (transform %in% c("both", "ms") && is.null(comp_b_ms)) next
    if (transform %in% c("both", "malmquist") && is.null(comp_b_mal)) next

    valid_count <- valid_count + 1

    if (transform %in% c("both", "ms")) {
      boot_ms_W2[valid_count] <- comp_b_ms$stat_W2
      boot_ms_A2[valid_count] <- comp_b_ms$stat_A2
    }
    if (transform %in% c("both", "malmquist")) {
      boot_mal_W2[valid_count] <- comp_b_mal$stat_W2
      boot_mal_A2[valid_count] <- comp_b_mal$stat_A2
    }
  }

  if (valid_count == 0) {
    stop("All bootstrap replicates failed during MLE estimation or standardization.")
  }

  p_vals <- list()
  if (transform %in% c("both", "ms")) {
    p_vals$mW2 <- mean(boot_ms_W2[1:valid_count] >= obs_ms$stat_W2)
    p_vals$mA2 <- mean(boot_ms_A2[1:valid_count] >= obs_ms$stat_A2)
  }
  if (transform %in% c("both", "malmquist")) {
    p_vals$lW2 <- mean(boot_mal_W2[1:valid_count] >= obs_mal$stat_W2)
    p_vals$lA2 <- mean(boot_mal_A2[1:valid_count] >= obs_mal$stat_A2)
  }

  list(
    p_values = p_vals,
    valid_replicates = valid_count,
    obs_ms = obs_ms,
    obs_malmquist = obs_mal
  )
}

#' Goodness-of-Fit Testing for Type-II Censored Samples
#'
#' Performs goodness-of-fit tests for arbitrary user-specified continuous parametric distributions
#' under Type-II right- or left-censoring using the Malmquist and Michael-Schucany transformations.
#'
#' @param data Numeric vector of observed order statistics, length r.
#' @param n Total sample size (n >= length(data)).
#' @param pdf Probability density function \code{f(x, params)}.
#' @param cdf Cumulative distribution function \code{F(x, params)}.
#' @param sf Survival function \code{S(x, params)}. If \code{NULL}, computed as \code{1 - cdf(x, params)}.
#' @param params Named list/vector of fixed parameter values (if \code{estimate = FALSE}) or starting values for MLE (if \code{estimate = TRUE}).
#' @param estimate Logical; \code{TRUE} for composite hypothesis (unknown parameters estimated via MLE), \code{FALSE} for simple hypothesis.
#' @param censoring Type of censoring: \code{"right"} or \code{"left"}.
#' @param transform Transformations to compute: \code{"both"}, \code{"malmquist"}, or \code{"ms"}.
#' @param statistics Subset of statistics to compute: \code{"all"}, \code{"direct7"}, \code{"complete_ad"}, \code{"composite_mle"}, or \code{"direct_censored"}.
#' @param pvalue_method Method for p-values: \code{"auto"}, \code{"distribution_free"}, or \code{"parametric_bootstrap"}.
#' @param nsim Number of Monte Carlo or bootstrap simulation replicates.
#' @param seed Optional random seed.
#' @param optim_control Optional control list passed to \code{\link[stats]{optim}}.
#'
#' @return An S3 object of class \code{"gofmalm"}.
#' @export
#'
#' @examples
#' # Simple hypothesis test on Uniform(0,1) with r=5, n=10
#' data_unif <- c(0.03, 0.06, 0.10, 0.11, 0.13)
#' res <- gof_censored(
#'   data = data_unif, n = 10,
#'   pdf = dunif, cdf = punif,
#'   estimate = FALSE, pvalue_method = "distribution_free",
#'   nsim = 500, seed = 123
#' )
#' print(res)
gof_censored <- function(data,
                         n,
                         pdf,
                         cdf,
                         sf = NULL,
                         params = NULL,
                         estimate = TRUE,
                         censoring = c("right", "left"),
                         transform = c("both", "malmquist", "ms"),
                         statistics = c("all", "direct7", "complete_ad", "composite_mle", "direct_censored"),
                         pvalue_method = c("auto", "distribution_free", "parametric_bootstrap"),
                         nsim = 2000,
                         seed = NULL,
                         optim_control = list()) {
  call_obj <- match.call()

  censoring <- match.arg(censoring)
  transform <- match.arg(transform)
  statistics <- match.arg(statistics)
  pvalue_method <- match.arg(pvalue_method)

  if (!is.numeric(data) || length(data) == 0) {
    stop("`data` must be a non-empty numeric vector.")
  }
  if (is.unsorted(data)) {
    stop("`data` must be sorted ascending.")
  }
  r <- length(data)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(data).")
  }
  if (!is.function(pdf) || !is.function(cdf)) {
    stop("`pdf` and `cdf` must be functions.")
  }
  if (is.null(sf)) {
    sf <- function(x, p = NULL) 1 - eval_func(cdf, x, p)
  }

  if (pvalue_method == "auto") {
    pvalue_method <- if (estimate) "parametric_bootstrap" else "distribution_free"
  }

  # Estimate or check parameters
  theta_hat <- params
  optim_conv <- 0

  if (estimate) {
    if (is.null(params) || length(params) == 0) {
      stop("`params` starting values must be supplied when `estimate = TRUE`.")
    }
    fit <- fit_mle(data = data, n = n, pdf = pdf, cdf = cdf, sf = sf,
                   params = params, censoring = censoring, optim_control = optim_control)
    theta_hat <- fit$par
    optim_conv <- fit$convergence
  } else {
    if (is.null(params)) {
      params <- list()
    }
  }

  # Probability Integral Transform (PIT)
  eps <- .Machine$double.eps^0.5
  u_pit <- tryCatch(eval_func(cdf, data, theta_hat), error = function(e) rep(NA_real_, r))
  if (any(is.na(u_pit))) {
    stop("Evaluation of `cdf(data, params)` returned NA/NaN values.")
  }
  u_pit <- pmin(pmax(u_pit, eps), 1 - eps)

  # Prepare uniform samples
  u_ms <- transform_ms(u_pit, n)
  u_malmquist <- transform_malmquist(u_pit, n, censoring = censoring)$W

  # Compute statistics & p-values
  stats_df <- data.frame(
    statistic = character(0),
    value = numeric(0),
    p_value = numeric(0),
    method = character(0),
    stringsAsFactors = FALSE
  )

  if (!estimate) {
    # Simple hypothesis mode
    edf_stats <- stat_edf_censored(u_pit, n, u_ms = u_ms, u_malmquist = u_malmquist)
    dir_stats <- stat_direct_censored(u_pit, n)
    all_raw <- c(edf_stats, dir_stats)

    p_vals <- pvalue_distribution_free(u_pit, n, r, censoring = censoring, nsim = nsim, seed = seed)

    stats_df <- data.frame(
      statistic = names(all_raw),
      value = as.numeric(all_raw),
      p_value = as.numeric(p_vals),
      method = "Monte Carlo (distribution-free)",
      stringsAsFactors = FALSE
    )
  } else {
    # Composite hypothesis mode
    comp_res <- pvalue_parametric_bootstrap(
      data = data, n = n, pdf = pdf, cdf = cdf, sf = sf,
      params_hat = theta_hat, censoring = censoring, transform = transform,
      nsim = nsim, seed = seed, optim_control = optim_control
    )

    rows <- list()
    if (transform %in% c("both", "ms")) {
      rows[[length(rows) + 1]] <- data.frame(
        statistic = "mW2",
        value = comp_res$obs_ms$stat_W2,
        p_value = comp_res$p_values$mW2,
        method = "Parametric Bootstrap (MS)",
        stringsAsFactors = FALSE
      )
      rows[[length(rows) + 1]] <- data.frame(
        statistic = "mA2",
        value = comp_res$obs_ms$stat_A2,
        p_value = comp_res$p_values$mA2,
        method = "Parametric Bootstrap (MS)",
        stringsAsFactors = FALSE
      )
    }
    if (transform %in% c("both", "malmquist")) {
      rows[[length(rows) + 1]] <- data.frame(
        statistic = "lW2",
        value = comp_res$obs_malmquist$stat_W2,
        p_value = comp_res$p_values$lW2,
        method = "Parametric Bootstrap (Malmquist)",
        stringsAsFactors = FALSE
      )
      rows[[length(rows) + 1]] <- data.frame(
        statistic = "lA2",
        value = comp_res$obs_malmquist$stat_A2,
        p_value = comp_res$p_values$lA2,
        method = "Parametric Bootstrap (Malmquist)",
        stringsAsFactors = FALSE
      )
    }

    stats_df <- do.call(rbind, rows)
  }

  res <- list(
    call = call_obj,
    n = n,
    r = r,
    censoring = censoring,
    estimate = estimate,
    theta_hat = theta_hat,
    optim_convergence = optim_conv,
    u_direct = u_pit,
    u_ms = u_ms,
    u_malmquist = u_malmquist,
    statistics = stats_df,
    settings = list(
      transform = transform,
      pvalue_method = pvalue_method,
      nsim = nsim
    )
  )

  class(res) <- "gofmalm"
  return(res)
}

#' @export
print.gofmalm <- function(x, ...) {
  cat("\n--- Goodness-of-Fit Test for Type-II Censored Samples ---\n\n")
  cat("Sample size (n):", x$n, "\n")
  cat("Observed failures (r):", x$r, "\n")
  cat("Censoring type:", x$censoring, "\n")
  cat("Hypothesis:", if (x$estimate) "Composite (parameters estimated)" else "Simple (fixed parameters)", "\n")

  if (x$estimate) {
    cat("\nEstimated Parameters:\n")
    print(unlist(x$theta_hat))
  }

  cat("\nTest Statistics:\n")
  print(x$statistics, row.names = FALSE)
  cat("\n")
  invisible(x)
}

#' @export
summary.gofmalm <- function(object, ...) {
  cat("\n=== Goodness-of-Fit Summary (gofmalm) ===\n")
  print.gofmalm(object, ...)
}

#' @export
plot.gofmalm <- function(x, ...) {
  r <- x$r
  theoretical_q <- (1:r - 0.5) / r

  old_par <- graphics::par(mfrow = c(1, 2))
  on.exit(graphics::par(old_par), add = TRUE)

  graphics::plot(theoretical_q, x$u_ms,
                 xlab = "Theoretical Uniform Quantiles",
                 ylab = "Transformed Sample (MS)",
                 main = "PP-Plot: Michael & Schucany",
                 pch = 19, col = "blue", xlim = c(0, 1), ylim = c(0, 1))
  graphics::abline(0, 1, lty = 2, col = "red")

  graphics::plot(theoretical_q, x$u_malmquist,
                 xlab = "Theoretical Uniform Quantiles",
                 ylab = "Transformed Sample (Malmquist)",
                 main = "PP-Plot: Malmquist",
                 pch = 19, col = "darkgreen", xlim = c(0, 1), ylim = c(0, 1))
  graphics::abline(0, 1, lty = 2, col = "red")

  invisible(x)
}

test_that("PIT boundary clipping and log(0) at i=0 work cleanly", {
  # Extreme u near 0 and 1
  u_ext <- c(1e-15, 0.2, 0.5, 0.8, 1 - 1e-15)
  n <- 10

  expect_silent({
    stats <- stat_edf_censored(u_ext, n)
  })

  expect_false(any(is.na(stats)))
  expect_false(any(is.nan(stats)))
  expect_false(any(is.infinite(stats)))
})

test_that("Malmquist transform handles boundary inputs without NaN or Inf", {
  u_bd <- c(1e-10, 0.5, 1 - 1e-10)
  n <- 5

  res <- transform_malmquist(u_bd, n, censoring = "right")
  expect_equal(length(res$W), 3)
  expect_false(any(is.na(res$W)))
  expect_false(any(is.infinite(res$W)))
})

test_that("Degenerate sample size r = 1 returns NA for T_rn and finite values for others", {
  u_single <- c(0.1)
  n <- 5

  stats <- stat_edf_censored(u_single, n)

  expect_true(is.na(stats["T_rn"]))
  expect_false(is.na(stats["D_rn"]))
  expect_false(is.na(stats["W2_rn"]))
})

test_that("Complete sample case r = n works without division by zero", {
  u_full <- c(0.1, 0.3, 0.5, 0.7, 0.9)
  n <- 5

  stats <- stat_edf_censored(u_full, n)

  expect_false(any(is.na(stats)))
  expect_false(any(is.nan(stats)))
})

test_that("Invalid inputs trigger clear error messages up front", {
  expect_error(stat_edf_censored(c(0.5, 0.2), n = 5), "sorted")
  expect_error(stat_edf_censored(c(0.1, 0.2), n = 1), "n` must be an integer >= length")
  expect_error(gof_censored(data = c(), n = 5, pdf = dunif, cdf = punif), "non-empty numeric vector")
})

test_that("Left censoring reduction runs correctly", {
  u_left <- c(0.6, 0.7, 0.8, 0.9)
  n <- 10

  res_mal <- transform_malmquist(u_left, n = 10, censoring = "left")
  expect_equal(length(res_mal$W), 4)
  expect_false(any(is.na(res_mal$W)))
})

#' Composite Hypothesis Procedure for Type-II Censored Samples
#'
#' Evaluates goodness-of-fit under unknown parameters using MLE estimation,
#' transformation to complete sample, normal residual standardization, and modified CvM/AD statistics.
#'
#' @param data Numeric vector of observed order statistics.
#' @param n Total sample size (n >= length(data)).
#' @param pdf Probability density function taking \code{(x, params)}.
#' @param cdf Cumulative distribution function taking \code{(x, params)}.
#' @param sf Survival function taking \code{(x, params)}. If \code{NULL}, computed as \code{1 - cdf(x, params)}.
#' @param params Named list/vector of starting parameter values for MLE.
#' @param censoring Type of censoring: \code{"right"} or \code{"left"}.
#' @param transform Transformation method: \code{"ms"} (Michael-Schucany) or \code{"malmquist"}.
#' @param optim_control Optional control list passed to \code{\link[stats]{optim}}.
#'
#' @return A list containing:
#' \item{stat_W2}{Modified Cramér-von Mises statistic (_mW2 or _lW2).}
#' \item{stat_A2}{Modified Anderson-Darling statistic (_mA2 or _lA2).}
#' \item{W2}{Raw Cramér-von Mises statistic on standardized residuals.}
#' \item{A2}{Raw Anderson-Darling statistic on standardized residuals.}
#' \item{theta_hat}{Estimated parameter values.}
#' \item{convergence}{Optimization convergence code.}
#' @export
#'
#' @examples
#' data_exp <- c(0.1, 0.2, 0.3, 0.4, 0.7, 1.0, 1.4)
#' stat_composite_mle(
#'   data = data_exp, n = 20,
#'   pdf = function(x, p) dexp(x, rate = p$rate),
#'   cdf = function(x, p) pexp(x, rate = p$rate),
#'   params = list(rate = 0.1),
#'   censoring = "right",
#'   transform = "malmquist"
#' )
stat_composite_mle <- function(data,
                               n,
                               pdf,
                               cdf,
                               sf = NULL,
                               params,
                               censoring = c("right", "left"),
                               transform = c("malmquist", "ms"),
                               optim_control = list()) {
  censoring <- match.arg(censoring)
  transform <- match.arg(transform)

  if (!is.numeric(data) || length(data) == 0) {
    stop("`data` must be a non-empty numeric vector.")
  }
  r <- length(data)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(data).")
  }

  if (is.null(sf)) {
    sf <- function(x, p = NULL) 1 - eval_func(cdf, x, p)
  }

  # Step 1: Compute MLE
  fit <- fit_mle(data = data, n = n, pdf = pdf, cdf = cdf, sf = sf, params = params,
                 censoring = censoring, optim_control = optim_control)
  theta_hat <- fit$par

  # Step 2: Probability integral transform v_i = F(x_i; theta_hat)
  eps <- .Machine$double.eps^0.5
  v_i <- eval_func(cdf, data, theta_hat)
  v_i <- pmin(pmax(v_i, eps), 1 - eps)

  if (transform == "ms") {
    v_star <- transform_ms(v_i, n)
  } else {
    v_star <- transform_malmquist(v_i, n, censoring = censoring)$W
  }

  v_star <- pmin(pmax(v_star, eps), 1 - eps)

  # Step 3: Normal probit transformation y_i = qnorm(v*_i)
  y <- stats::qnorm(v_star)

  # Step 4: Standardize
  y_bar <- mean(y)
  s_y2 <- stats::var(y)

  if (is.na(s_y2) || s_y2 <= .Machine$double.eps) {
    stop("Sample variance of normal residuals is zero or undefined (s_y <= 0). Cannot standardize.")
  }

  s_y <- sqrt(s_y2)
  u_std <- stats::pnorm((y - y_bar) / s_y)
  u_std <- sort(pmin(pmax(u_std, eps), 1 - eps))

  # Step 5: Compute W² and A²
  i_seq <- 1:r
  W2 <- sum((u_std - (2 * i_seq - 1) / (2 * r))^2) + 1 / (12 * r)
  A2 <- -r - (1 / r) * sum((2 * i_seq - 1) * log(u_std) + (2 * r + 1 - 2 * i_seq) * log(1 - u_std))

  # Step 6: Modification factors
  mod_W2 <- W2 * (1 + 0.5 / r)
  mod_A2 <- A2 * (1 + 0.75 / r + 2.25 / (r^2))

  list(
    stat_W2 = mod_W2,
    stat_A2 = mod_A2,
    W2 = W2,
    A2 = A2,
    theta_hat = theta_hat,
    convergence = fit$convergence
  )
}

eval_func <- function(fn, x, params) {
  if (is.null(fn)) return(NULL)
  if (is.null(params) || length(params) == 0) {
    res <- tryCatch(fn(x), error = function(e) NULL)
    if (!is.null(res)) return(res)
  }
  res <- tryCatch(fn(x, params), error = function(e) NULL)
  if (!is.null(res) && !any(is.na(res)) && length(res) == length(x)) {
    return(res)
  }
  res2 <- tryCatch(do.call(fn, c(list(x), params)), error = function(e) NULL)
  if (!is.null(res2) && !any(is.na(res2)) && length(res2) == length(x)) {
    return(res2)
  }
  stop("Failed to evaluate function with supplied parameters.")
}

#' Maximum Likelihood Estimation for Type-II Censored Samples
#'
#' Estimates distribution parameters under Type-II right- or left-censoring.
#'
#' @param data Numeric vector of length \code{r}, sorted ascending.
#' @param n Total sample size (\code{n >= r}).
#' @param pdf Probability density function.
#' @param cdf Cumulative distribution function.
#' @param sf Survival function. If \code{NULL}, computed as \code{1 - cdf(x, params)}.
#' @param params Named list or vector of starting parameter values.
#' @param censoring Type of censoring: \code{"right"} or \code{"left"}.
#' @param optim_control Optional control list passed to \code{\link[stats]{optim}}.
#'
#' @return A list containing \code{par} (estimated parameters), \code{value} (negative log-likelihood),
#' \code{convergence} status code, and \code{message}.
#' @export
#'
#' @examples
#' data_exp <- c(0.1, 0.2, 0.3, 0.4, 0.7, 1.0, 1.4)
#' fit <- fit_mle(
#'   data = data_exp, n = 20,
#'   pdf = function(x, p) dexp(x, rate = p$rate),
#'   cdf = function(x, p) pexp(x, rate = p$rate),
#'   params = list(rate = 0.1),
#'   censoring = "right"
#' )
#' fit$par
fit_mle <- function(data,
                    n,
                    pdf,
                    cdf,
                    sf = NULL,
                    params,
                    censoring = c("right", "left"),
                    optim_control = list()) {
  censoring <- match.arg(censoring)
  if (!is.numeric(data) || length(data) == 0) {
    stop("`data` must be a non-empty numeric vector.")
  }
  if (is.unsorted(data)) {
    stop("`data` must be sorted ascending.")
  }
  r <- length(data)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(data).")
  }
  if (!is.function(pdf) || !is.function(cdf)) {
    stop("`pdf` and `cdf` must be functions.")
  }
  if (is.null(sf)) {
    sf <- function(x, p = NULL) 1 - eval_func(cdf, x, p)
  }
  if (is.null(params) || length(params) == 0) {
    stop("`params` must be a non-empty list or vector of parameter values.")
  }

  par_names <- names(params)
  if (is.null(par_names) || any(par_names == "")) {
    stop("`params` must be named.")
  }
  start_vec <- unlist(params)

  eps <- .Machine$double.eps^0.5

  nll <- function(par) {
    p_list <- as.list(par)
    names(p_list) <- par_names

    dens <- tryCatch(eval_func(pdf, data, p_list), error = function(e) rep(NA_real_, r))
    if (any(is.na(dens)) || any(dens <= 0)) {
      return(.Machine$double.xmax)
    }

    if (censoring == "right") {
      surv_r <- tryCatch(eval_func(sf, data[r], p_list), error = function(e) NA_real_)
      if (is.na(surv_r) || surv_r <= 0) {
        return(.Machine$double.xmax)
      }
      surv_r <- max(surv_r, eps)
      dens <- pmax(dens, eps)
      log_lik <- sum(log(dens)) + (n - r) * log(surv_r)
    } else {
      cdf_1 <- tryCatch(eval_func(cdf, data[1], p_list), error = function(e) NA_real_)
      if (is.na(cdf_1) || cdf_1 <= 0) {
        return(.Machine$double.xmax)
      }
      cdf_1 <- max(cdf_1, eps)
      dens <- pmax(dens, eps)
      log_lik <- (n - r) * log(cdf_1) + sum(log(dens))
    }

    if (!is.finite(log_lik)) {
      return(.Machine$double.xmax)
    }
    return(-log_lik)
  }

  opt <- stats::optim(par = start_vec, fn = nll, control = optim_control)

  if (opt$convergence != 0) {
    warning("MLE optimization did not converge (convergence code: ", opt$convergence, ").")
  }

  res_par <- as.list(opt$par)
  names(res_par) <- par_names

  list(
    par = res_par,
    value = opt$value,
    convergence = opt$convergence,
    message = opt$message
  )
}

#' Michael & Schucany (1979) Transformation
#'
#' Transforms an r-out-of-n Type-II right-censored uniform sample into a complete sample of size r.
#'
#' @param u_cens Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).
#' @param n Total sample size (n >= length(u_cens)).
#'
#' @return Numeric vector of length r representing the transformed complete sample Z_1:r < ... < Z_r:r.
#' @export
#'
#' @examples
#' u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
#' transform_ms(u, n = 10)
transform_ms <- function(u_cens, n) {
  if (!is.numeric(u_cens) || length(u_cens) == 0) {
    stop("`u_cens` must be a non-empty numeric vector.")
  }
  r <- length(u_cens)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(u_cens).")
  }

  eps <- .Machine$double.eps^0.5
  u_cens <- pmin(pmax(u_cens, eps), 1 - eps)
  u_r <- u_cens[r]

  pbeta_val <- stats::pbeta(u_r, shape1 = r, shape2 = n - r + 1)
  pbeta_val <- pmin(pmax(pbeta_val, eps), 1 - eps)

  h_rn <- (pbeta_val^(1 / r)) / u_r
  z <- u_cens * h_rn
  z_sorted <- sort(pmin(pmax(z, eps), 1 - eps))
  return(z_sorted)
}

#' Compute EDF Statistics for Type-II Censored Samples
#'
#' Computes the nine EDF test statistics described in Lin, Huang & Balakrishnan (2008).
#'
#' @param u_cens Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).
#' @param n Total sample size (n >= length(u_cens)).
#' @param u_ms Optional pre-computed Michael & Schucany transformed sample.
#' @param u_malmquist Optional pre-computed Malmquist transformed sample.
#'
#' @return A named numeric vector of 9 statistics: \code{D_rn}, \code{G_rn}, \code{T_rn},
#' \code{W2_star_rn}, \code{W2_rn}, \code{A2_rn}, \code{U2_rn}, \code{TA2_r}, and \code{star_TA2_r}.
#' @export
#'
#' @examples
#' u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
#' stat_edf_censored(u, n = 10)
stat_edf_censored <- function(u_cens, n, u_ms = NULL, u_malmquist = NULL) {
  if (!is.numeric(u_cens) || length(u_cens) == 0) {
    stop("`u_cens` must be a non-empty numeric vector.")
  }
  if (is.unsorted(u_cens)) {
    stop("`u_cens` must be sorted ascending.")
  }
  r <- length(u_cens)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(u_cens).")
  }

  eps <- .Machine$double.eps^0.5
  u_cens <- pmin(pmax(u_cens, eps), 1 - eps)

  i_seq <- 1:r

  # 1. Modified Kolmogorov-Smirnov (D_{r,n})
  term1 <- abs(i_seq / n - u_cens)
  term2 <- abs((i_seq - 1) / n - u_cens)
  D_rn <- max(max(term1), max(term2))

  # 2. Squared-spacings statistic (G_{r,n})
  u_prev <- c(0, u_cens[1:(r - 1)])
  sum_sq_spacings <- sum((u_cens - u_prev)^2)
  u_r <- u_cens[r]
  term_end <- ((1 - u_r)^2) / (n - r + 1)
  G_rn <- (n + 1) * (n + 2) * (sum_sq_spacings + term_end) - (n + 2)

  # 3. Modified correlation statistic (T_{r,n})
  if (r == 1) {
    T_rn <- NA_real_
  } else {
    r_cor <- stats::cor(u_cens, i_seq)
    T_rn <- 1 - r_cor^2
  }

  # 4. Modified Cramér-von Mises star form (*W²_{r,n})
  W2_star_rn <- 1 / (12 * n) + sum((u_cens - (2 * i_seq - 1) / (2 * n))^2)

  # 5. Modified Cramér-von Mises integral form (W²_{r,n})
  u_full <- c(0, u_cens)
  i_idx <- 0:(r - 1)
  term_w2 <- (i_idx / n - u_full[i_idx + 1])^3 - (i_idx / n - u_full[i_idx + 2])^3
  W2_rn <- (n / 3) * sum(term_w2)

  # 6. Modified Anderson-Darling integral form (A²_{r,n})
  term_ad <- numeric(r)
  # i = 0 special case: (0/n)^2 * ln(...) is 0
  term_ad[1] <- - (1)^2 * (log(1 - u_full[2]) - log(1 - 0)) - (u_full[2] - 0)
  if (r > 1) {
    for (idx in 2:r) {
      i_val <- idx - 1
      in_ratio <- i_val / n
      u_curr <- u_full[idx]
      u_next <- u_full[idx + 1]
      term_ad[idx] <- (in_ratio^2) * (log(u_next) - log(u_curr)) -
        ((1 - in_ratio)^2) * (log(1 - u_next) - log(1 - u_curr)) -
        (u_next - u_curr)
    }
  }
  A2_rn <- n * sum(term_ad)

  # 7. Modified Watson statistic (U²_{r,n})
  sum_u <- sum(u_cens)
  bracket <- (r / n) - (u_r / 2) - (sum_u / (n * u_r))
  U2_rn <- W2_rn - n * u_r * (bracket^2)

  # Helper for complete AD statistic
  calc_ad_complete <- function(u_comp) {
    u_comp <- sort(pmin(pmax(u_comp, eps), 1 - eps))
    -r - (1 / r) * sum((2 * i_seq - 1) * log(u_comp) + (2 * r + 1 - 2 * i_seq) * log(1 - u_comp))
  }

  # 8. AD on Transformation A (Michael & Schucany)
  if (is.null(u_ms)) {
    u_ms <- transform_ms(u_cens, n)
  }
  TA2_r <- calc_ad_complete(u_ms)

  # 9. AD on Transformation B (Malmquist)
  if (is.null(u_malmquist)) {
    u_malmquist <- transform_malmquist(u_cens, n, censoring = "right")$W
  }
  star_TA2_r <- calc_ad_complete(u_malmquist)

  c(
    D_rn = D_rn,
    G_rn = G_rn,
    T_rn = T_rn,
    W2_star_rn = W2_star_rn,
    W2_rn = W2_rn,
    A2_rn = A2_rn,
    U2_rn = U2_rn,
    TA2_r = TA2_r,
    star_TA2_r = star_TA2_r
  )
}

#' Compute Direct Censored EDF Statistics (Pettitt & Stephens, 1976)
#'
#' Computes the untransformed direct censored Cramér-von Mises and Anderson-Darling statistics.
#'
#' @param u_cens Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).
#' @param n Total sample size (n >= length(u_cens)).
#'
#' @return Named numeric vector containing \code{W2_2_rn} and \code{A2_2_rn}.
#' @export
#'
#' @examples
#' u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
#' stat_direct_censored(u, n = 10)
stat_direct_censored <- function(u_cens, n) {
  if (!is.numeric(u_cens) || length(u_cens) == 0) {
    stop("`u_cens` must be a non-empty numeric vector.")
  }
  if (is.unsorted(u_cens)) {
    stop("`u_cens` must be sorted ascending.")
  }
  r <- length(u_cens)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(u_cens).")
  }

  eps <- .Machine$double.eps^0.5
  v <- pmin(pmax(u_cens, eps), 1 - eps)
  v_r <- v[r]
  i_seq <- 1:r

  W2_2_rn <- sum((v - (2 * i_seq - 1) / (2 * n))^2) + (r / (12 * n^2)) + (n / 3) * ((v_r - r / n)^3)

  t1 <- -(1 / n) * sum((2 * i_seq - 1) * (log(v) - log(1 - v)))
  t2 <- -2 * sum(log(1 - v))
  t3 <- -(1 / n) * (((r - n)^2) * log(1 - v_r) - (r^2) * log(v_r) + (n^2) * v_r)

  A2_2_rn <- t1 + t2 + t3

  c(W2_2_rn = W2_2_rn, A2_2_rn = A2_2_rn)
}

test_that("Closed-form W2_rn matches numerical integration across intervals", {
  u <- c(0.05, 0.12, 0.25, 0.40, 0.55)
  n <- 10
  r <- 5

  # Closed form
  stats_cf <- stat_edf_censored(u, n)
  w2_closed <- unname(stats_cf["W2_rn"])

  # Piecewise numerical integration per interval
  u_full <- c(0, u)
  w2_num_sum <- 0
  for (i in 0:(r - 1)) {
    i_n <- i / n
    u_a <- u_full[i + 1]
    u_b <- u_full[i + 2]
    val <- stats::integrate(function(uv) (i_n - uv)^2, lower = u_a, upper = u_b)$value
    w2_num_sum <- w2_num_sum + val
  }
  w2_num <- n * w2_num_sum

  expect_equal(w2_closed, w2_num, tolerance = 1e-6)
})

test_that("Closed-form A2_rn matches numerical integration across intervals", {
  u <- c(0.05, 0.12, 0.25, 0.40, 0.55)
  n <- 10
  r <- 5

  # Closed form
  stats_cf <- stat_edf_censored(u, n)
  a2_closed <- unname(stats_cf["A2_rn"])

  # Piecewise numerical integration per interval
  u_full <- c(0, u)
  a2_num_sum <- 0
  for (i in 0:(r - 1)) {
    i_n <- i / n
    u_a <- max(u_full[i + 1], 1e-7)
    u_b <- min(u_full[i + 2], 1 - 1e-7)
    val <- stats::integrate(function(uv) ((i_n - uv)^2) / (uv * (1 - uv)), lower = u_a, upper = u_b)$value
    a2_num_sum <- a2_num_sum + val
  }
  a2_num <- n * a2_num_sum

  expect_equal(a2_closed, a2_num, tolerance = 1e-3)
})

test_that("Example 1 matches paper's expected values (Law & Kelton data)", {
  data_ex1 <- c(0.03, 0.06, 0.10, 0.11, 0.13)
  n <- 10
  r <- 5

  u <- data_ex1

  stats <- stat_edf_censored(u, n)

  expect_equal(round(unname(stats["D_rn"]), 3), 0.370)
  expect_equal(round(unname(stats["G_rn"]), 3), 5.167)
  expect_equal(round(unname(stats["T_rn"]), 3), 0.041)
  expect_equal(round(unname(stats["W2_star_rn"]), 3), 0.199)
  expect_equal(round(unname(stats["W2_rn"]), 3), 0.026)
  expect_equal(round(unname(stats["A2_rn"]), 3), 0.301)
  expect_equal(round(unname(stats["U2_rn"]), 3), 0.012)
  expect_equal(round(unname(stats["TA2_r"]), 3), 2.505)
  expect_equal(round(unname(stats["star_TA2_r"]), 3), 3.098)
})

test_that("Example 2 matches paper's expected values (Barr & Davidson data)", {
  data_ex2 <- c(0.1, 0.2, 0.3, 0.4, 0.7, 1.0, 1.4)
  n <- 20
  r <- 7

  rate_known <- 0.1
  u <- pexp(data_ex2, rate = rate_known)

  stats <- stat_edf_censored(u, n)

  expect_equal(round(unname(stats["TA2_r"]), 3), 3.404)
  expect_equal(round(unname(stats["star_TA2_r"]), 3), 3.042)
})

#' Malmquist (1950) Transformation for Type-II Censored Samples
#'
#' Converts an r-out-of-n Type-II right- or left-censored uniform sample into an independent
#' complete Uniform(0,1) sample of size r (Lin, Huang & Balakrishnan, 2008).
#'
#' @param u_cens Numeric vector of length r containing PIT-transformed order statistics.
#' @param n Total sample size (n >= length(u_cens)).
#' @param censoring Type of censoring: \code{"right"} or \code{"left"}.
#'
#' @return A list containing:
#' \item{W}{Sorted complete sample W_1:r < ... < W_r:r of size r.}
#' \item{V}{Vector of intermediate Beta-distributed random variables V_n, ..., V_\{n-r+1\}.}
#' @export
#'
#' @examples
#' u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
#' transform_malmquist(u, n = 10, censoring = "right")
transform_malmquist <- function(u_cens, n, censoring = c("right", "left")) {
  censoring <- match.arg(censoring)
  if (!is.numeric(u_cens) || length(u_cens) == 0) {
    stop("`u_cens` must be a non-empty numeric vector.")
  }
  if (is.unsorted(u_cens)) {
    stop("`u_cens` must be sorted ascending.")
  }
  r <- length(u_cens)
  if (!is.numeric(n) || length(n) != 1 || n < r) {
    stop("`n` must be an integer >= length(u_cens).")
  }

  eps <- .Machine$double.eps^0.5

  if (censoring == "left") {
    # Left-censoring reduction: u*_i:n = 1 - u_{n+1-i:n}
    u_cens <- 1 - u_cens[r:1]
  }

  u_cens <- pmin(pmax(u_cens, eps), 1 - eps)

  v <- numeric(r)
  v[1] <- 1 - u_cens[1]

  if (r > 1) {
    for (i in 1:(r - 1)) {
      denom <- 1 - u_cens[i]
      denom <- max(denom, eps)
      v[i + 1] <- (1 - u_cens[i + 1]) / denom
    }
  }

  v <- pmin(pmax(v, eps), 1 - eps)

  # W_i = V_{n-i+1}^{n-i+1} for i = 1, ..., r
  k_powers <- n:(n - r + 1)
  w <- v^k_powers

  w <- pmin(pmax(w, eps), 1 - eps)
  w_sorted <- sort(w)

  list(
    W = w_sorted,
    V = v
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/composite_mle_procedure.R
\name{stat_composite_mle}
\alias{stat_composite_mle}
\title{Composite Hypothesis Procedure for Type-II Censored Samples}
\usage{
stat_composite_mle(
  data,
  n,
  pdf,
  cdf,
  sf = NULL,
  params,
  censoring = c("right", "left"),
  transform = c("malmquist", "ms"),
  optim_control = list()
)
}
\arguments{
\item{data}{Numeric vector of observed order statistics.}

\item{n}{Total sample size (n >= length(data)).}

\item{pdf}{Probability density function taking \code{(x, params)}.}

\item{cdf}{Cumulative distribution function taking \code{(x, params)}.}

\item{sf}{Survival function taking \code{(x, params)}. If \code{NULL}, computed as \code{1 - cdf(x, params)}.}

\item{params}{Named list/vector of starting parameter values for MLE.}

\item{censoring}{Type of censoring: \code{"right"} or \code{"left"}.}

\item{transform}{Transformation method: \code{"ms"} (Michael-Schucany) or \code{"malmquist"}.}

\item{optim_control}{Optional control list passed to \code{\link[stats]{optim}}.}
}
\value{
A list containing:
\item{stat_W2}{Modified Cramér-von Mises statistic (_mW2 or _lW2).}
\item{stat_A2}{Modified Anderson-Darling statistic (_mA2 or _lA2).}
\item{W2}{Raw Cramér-von Mises statistic on standardized residuals.}
\item{A2}{Raw Anderson-Darling statistic on standardized residuals.}
\item{theta_hat}{Estimated parameter values.}
\item{convergence}{Optimization convergence code.}
}
\description{
Evaluates goodness-of-fit under unknown parameters using MLE estimation,
transformation to complete sample, normal residual standardization, and modified CvM/AD statistics.
}
\examples{
data_exp <- c(0.1, 0.2, 0.3, 0.4, 0.7, 1.0, 1.4)
stat_composite_mle(
  data = data_exp, n = 20,
  pdf = function(x, p) dexp(x, rate = p$rate),
  cdf = function(x, p) pexp(x, rate = p$rate),
  params = list(rate = 0.1),
  censoring = "right",
  transform = "malmquist"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalues.R
\name{pvalue_distribution_free}
\alias{pvalue_distribution_free}
\title{Compute Distribution-Free p-values for Simple Hypotheses}
\usage{
pvalue_distribution_free(
  u_obs,
  n,
  r,
  censoring = c("right", "left"),
  nsim = 2000,
  seed = NULL
)
}
\arguments{
\item{u_obs}{Observed order statistics u_1:n < ... < u_r:n.}

\item{n}{Total sample size.}

\item{r}{Number of observed failures.}

\item{censoring}{Type of censoring: \code{"right"} or \code{"left"}.}

\item{nsim}{Number of Monte Carlo simulation replicates (default 2000).}

\item{seed}{Optional random seed.}
}
\value{
Named numeric vector of empirical p-values for each statistic.
}
\description{
Obtains p-values for EDF goodness-of-fit statistics via Monte Carlo simulation
from Uniform(0,1) order statistics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gofmalm-package.R
\docType{package}
\name{gofmalm-package}
\alias{gofmalm}
\alias{gofmalm-package}
\title{gofmalm: Goodness-of-Fit Tests for Type-II Censored Samples via the Malmquist Transformation}
\description{
Goodness-of-fit tests for an arbitrary user-specified continuous distribution under Type-II right- or left-censoring. Implements the transformation-based method of Lin, Huang and Balakrishnan (2008) \doi{10.1109/TR.2008.2005860}, which uses a property of order statistics due to Malmquist (1950) to convert an r-out-of-n Type-II censored uniform sample into a complete sample of size r, alongside the earlier transformation of Michael and Schucany (1979) \doi{10.1080/00401706.1979.10489813}. Also implements the direct (untransformed) censored-sample statistics of Barr and Davidson (1973) \doi{10.1080/00401706.1973.10489108} and Pettitt and Stephens (1976) \doi{10.1093/biomet/63.2.291}, and the modified-statistic maximum-likelihood procedure of Chen and Balakrishnan (1995) for testing composite hypotheses. General background on empirical-distribution-function goodness-of-fit methods follows D'Agostino and Stephens (1986, ISBN:978-0-8247-7487-5).
}
\author{
\strong{Maintainer}: Shikhar Tyagi \email{shikhar1093tyagi@gmail.com} (\href{https://orcid.org/0000-0003-1606-0844}{ORCID})

Authors:
\itemize{
  \item Arvind Pandey \email{arvindmzu@gmail.com}
  \item Bhupendra Singh \email{bhupendra.rana@gmail.com}
  \item Vrijesh Tripathi \email{vrijesh.tripathi@uwi.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_malmquist.R
\name{transform_malmquist}
\alias{transform_malmquist}
\title{Malmquist (1950) Transformation for Type-II Censored Samples}
\usage{
transform_malmquist(u_cens, n, censoring = c("right", "left"))
}
\arguments{
\item{u_cens}{Numeric vector of length r containing PIT-transformed order statistics.}

\item{n}{Total sample size (n >= length(u_cens)).}

\item{censoring}{Type of censoring: \code{"right"} or \code{"left"}.}
}
\value{
A list containing:
\item{W}{Sorted complete sample W_1:r < ... < W_r:r of size r.}
\item{V}{Vector of intermediate Beta-distributed random variables V_n, ..., V_\{n-r+1\}.}
}
\description{
Converts an r-out-of-n Type-II right- or left-censored uniform sample into an independent
complete Uniform(0,1) sample of size r (Lin, Huang & Balakrishnan, 2008).
}
\examples{
u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
transform_malmquist(u, n = 10, censoring = "right")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{stat_edf_censored}
\alias{stat_edf_censored}
\title{Compute EDF Statistics for Type-II Censored Samples}
\usage{
stat_edf_censored(u_cens, n, u_ms = NULL, u_malmquist = NULL)
}
\arguments{
\item{u_cens}{Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).}

\item{n}{Total sample size (n >= length(u_cens)).}

\item{u_ms}{Optional pre-computed Michael & Schucany transformed sample.}

\item{u_malmquist}{Optional pre-computed Malmquist transformed sample.}
}
\value{
A named numeric vector of 9 statistics: \code{D_rn}, \code{G_rn}, \code{T_rn},
\code{W2_star_rn}, \code{W2_rn}, \code{A2_rn}, \code{U2_rn}, \code{TA2_r}, and \code{star_TA2_r}.
}
\description{
Computes the nine EDF test statistics described in Lin, Huang & Balakrishnan (2008).
}
\examples{
u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
stat_edf_censored(u, n = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_ms.R
\name{transform_ms}
\alias{transform_ms}
\title{Michael & Schucany (1979) Transformation}
\usage{
transform_ms(u_cens, n)
}
\arguments{
\item{u_cens}{Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).}

\item{n}{Total sample size (n >= length(u_cens)).}
}
\value{
Numeric vector of length r representing the transformed complete sample Z_1:r < ... < Z_r:r.
}
\description{
Transforms an r-out-of-n Type-II right-censored uniform sample into a complete sample of size r.
}
\examples{
u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
transform_ms(u, n = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof_censored.R
\name{gof_censored}
\alias{gof_censored}
\title{Goodness-of-Fit Testing for Type-II Censored Samples}
\usage{
gof_censored(
  data,
  n,
  pdf,
  cdf,
  sf = NULL,
  params = NULL,
  estimate = TRUE,
  censoring = c("right", "left"),
  transform = c("both", "malmquist", "ms"),
  statistics = c("all", "direct7", "complete_ad", "composite_mle", "direct_censored"),
  pvalue_method = c("auto", "distribution_free", "parametric_bootstrap"),
  nsim = 2000,
  seed = NULL,
  optim_control = list()
)
}
\arguments{
\item{data}{Numeric vector of observed order statistics, length r.}

\item{n}{Total sample size (n >= length(data)).}

\item{pdf}{Probability density function \code{f(x, params)}.}

\item{cdf}{Cumulative distribution function \code{F(x, params)}.}

\item{sf}{Survival function \code{S(x, params)}. If \code{NULL}, computed as \code{1 - cdf(x, params)}.}

\item{params}{Named list/vector of fixed parameter values (if \code{estimate = FALSE}) or starting values for MLE (if \code{estimate = TRUE}).}

\item{estimate}{Logical; \code{TRUE} for composite hypothesis (unknown parameters estimated via MLE), \code{FALSE} for simple hypothesis.}

\item{censoring}{Type of censoring: \code{"right"} or \code{"left"}.}

\item{transform}{Transformations to compute: \code{"both"}, \code{"malmquist"}, or \code{"ms"}.}

\item{statistics}{Subset of statistics to compute: \code{"all"}, \code{"direct7"}, \code{"complete_ad"}, \code{"composite_mle"}, or \code{"direct_censored"}.}

\item{pvalue_method}{Method for p-values: \code{"auto"}, \code{"distribution_free"}, or \code{"parametric_bootstrap"}.}

\item{nsim}{Number of Monte Carlo or bootstrap simulation replicates.}

\item{seed}{Optional random seed.}

\item{optim_control}{Optional control list passed to \code{\link[stats]{optim}}.}
}
\value{
An S3 object of class \code{"gofmalm"}.
}
\description{
Performs goodness-of-fit tests for arbitrary user-specified continuous parametric distributions
under Type-II right- or left-censoring using the Malmquist and Michael-Schucany transformations.
}
\examples{
# Simple hypothesis test on Uniform(0,1) with r=5, n=10
data_unif <- c(0.03, 0.06, 0.10, 0.11, 0.13)
res <- gof_censored(
  data = data_unif, n = 10,
  pdf = dunif, cdf = punif,
  estimate = FALSE, pvalue_method = "distribution_free",
  nsim = 500, seed = 123
)
print(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/statistics.R
\name{stat_direct_censored}
\alias{stat_direct_censored}
\title{Compute Direct Censored EDF Statistics (Pettitt & Stephens, 1976)}
\usage{
stat_direct_censored(u_cens, n)
}
\arguments{
\item{u_cens}{Numeric vector of order statistics u_1:n < ... < u_r:n in (0, 1).}

\item{n}{Total sample size (n >= length(u_cens)).}
}
\value{
Named numeric vector containing \code{W2_2_rn} and \code{A2_2_rn}.
}
\description{
Computes the untransformed direct censored Cramér-von Mises and Anderson-Darling statistics.
}
\examples{
u <- c(0.03, 0.06, 0.10, 0.11, 0.13)
stat_direct_censored(u, n = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalues.R
\name{pvalue_parametric_bootstrap}
\alias{pvalue_parametric_bootstrap}
\title{Compute Parametric Bootstrap p-values for Composite Hypotheses}
\usage{
pvalue_parametric_bootstrap(
  data,
  n,
  pdf,
  cdf,
  sf = NULL,
  params_hat,
  censoring = c("right", "left"),
  transform = c("both", "malmquist", "ms"),
  nsim = 2000,
  seed = NULL,
  optim_control = list()
)
}
\arguments{
\item{data}{Observed data vector of length r.}

\item{n}{Total sample size.}

\item{pdf}{Density function.}

\item{cdf}{Distribution function.}

\item{sf}{Survival function.}

\item{params_hat}{Estimated parameter list from observed data.}

\item{censoring}{Type of censoring.}

\item{transform}{Transformation method (\code{"ms"}, \code{"malmquist"}, or \code{"both"}).}

\item{nsim}{Number of bootstrap replicates (default 2000).}

\item{seed}{Optional random seed.}

\item{optim_control}{Optional control list passed to \code{optim}.}
}
\value{
A list containing empirical p-values and number of valid replicates.
}
\description{
Obtains p-values for composite hypothesis tests where parameters are estimated from data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle.R
\name{fit_mle}
\alias{fit_mle}
\title{Maximum Likelihood Estimation for Type-II Censored Samples}
\usage{
fit_mle(
  data,
  n,
  pdf,
  cdf,
  sf = NULL,
  params,
  censoring = c("right", "left"),
  optim_control = list()
)
}
\arguments{
\item{data}{Numeric vector of length \code{r}, sorted ascending.}

\item{n}{Total sample size (\code{n >= r}).}

\item{pdf}{Probability density function.}

\item{cdf}{Cumulative distribution function.}

\item{sf}{Survival function. If \code{NULL}, computed as \code{1 - cdf(x, params)}.}

\item{params}{Named list or vector of starting parameter values.}

\item{censoring}{Type of censoring: \code{"right"} or \code{"left"}.}

\item{optim_control}{Optional control list passed to \code{\link[stats]{optim}}.}
}
\value{
A list containing \code{par} (estimated parameters), \code{value} (negative log-likelihood),
\code{convergence} status code, and \code{message}.
}
\description{
Estimates distribution parameters under Type-II right- or left-censoring.
}
\examples{
data_exp <- c(0.1, 0.2, 0.3, 0.4, 0.7, 1.0, 1.4)
fit <- fit_mle(
  data = data_exp, n = 20,
  pdf = function(x, p) dexp(x, rate = p$rate),
  cdf = function(x, p) pexp(x, rate = p$rate),
  params = list(rate = 0.1),
  censoring = "right"
)
fit$par
}
